"""
======
update-checker
======

code from:
https://github.com/bboe/update_checker
package:
https://pypi.org/project/update-checker/

Public resources from ``update-checker``
"""

from checkov.common.util.update_checker.update_checker import UpdateChecker

__all__ = ("UpdateChecker",)
